%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef YYSTYPE
#define YYSTYPE char *
#endif

char* result = "";

int yylex(void);
int yyerror(char* s);

%}

%token VALUE
%token REGISTER REGISTER_REF OP OP_CTRL
%token PAR_O PAR_C
%left COMMA
%right SEPARATION

%%
program:
    input SEPARATION instruction {
        asprintf(&result, "input = [%s]\ninstructions = [ %s]\n", $1, $3);
        }
    |   instruction {
        asprintf(&result, "input = []\ninstructions = [ %s]\n", $1);
        }
;

input:
    PAR_O content PAR_C {
        $$ = $2;
        }
;

content:
    content COMMA VALUE {
        char* tmp = "";
        asprintf(&tmp, "%s, %s", $1, $3);
        $$ = tmp;
        }
    |   VALUE {
        $$ = $1;
        }
;

instruction:
    line SEPARATION instruction {
        char* tmp = "";
        asprintf(&tmp, "%s%s", $1, $3);
        $$ = tmp;
        }
    |   line SEPARATION {
        $$ = $1;
        }
    |   line {
        $$ = $1;
        }
;

line:
    OP_CTRL PAR_O args_ctrl PAR_C {
        char* tmp = "";
        asprintf(&tmp, "{\"op\": Ram.op_ctrl, \"args\": ('%s', (%s))},\n", $1, $3);
        $$ = tmp;
        }
    |   OP PAR_O args PAR_C {
        char* tmp = "";
        asprintf(&tmp, "{\"op\": Ram.op, \"args\": ('%s', (%s))},\n", $1, $3);
        $$ = tmp;
        }
;
args:
    arg COMMA arg COMMA arg {
        char* tmp = "";
        asprintf(&tmp, "%s, %s, %s", $1, $3, $5);
        $$ = tmp;
        }
    |   arg {
        $$ = $1;
        }
;

args_ctrl:
    VALUE COMMA arg COMMA arg {
        char* tmp = "";
        asprintf(&tmp, "%c, %s, %s", $1, $3, $5);
        $$ = tmp;
        }
    |   VALUE {
        $$ = $1;
        }

arg:
    REGISTER_REF {
        char* tmp = "";
        char *index_str = strdup($1 + 3); // copy from 4th char
        asprintf(&tmp, "('%c', '%c', '%c', %s)", $1[1], index_str, $1[2], $1[0]);
        // r@i1 -> r register, index in i1
        // Python args: type_register, index, ref_origin, ref_target
        // r@i1 -> ('@', '1', 'i', 'r')
        free(index_str);
        $$ = tmp;
        }
    |   REGISTER {
        char* tmp = "";
        char *index_str = strdup($1 + 1); // copy from 2nd char
        asprintf(&tmp, "('%c', %s)", $1[0], index_str);
        free(index_str);
        $$ = tmp;
        }
    |   VALUE {
        char* tmp = "";
        asprintf(&tmp, "('value', %s)", $1);
        $$ = tmp;
        }
;

%%

int main() {
    yyparse();
    FILE *file = fopen("machine.py", "w");

    if (file == NULL) {
        fprintf(stderr, "Error creating file machine.py\n");
        return 1;
    } else {
        fprintf(file, "%s", result);
        fclose(file);
    }
    return 0;
}
